AC_DEFUN([AC_ARG_FFTW],
	[
		AC_ARG_WITH(
			[fftw],
			[AS_HELP_STRING([--with-fftw@<:@=prefix@:>@], [specify the location of FFTW @<:@default=check@:>@])],
			[ac_cv_use_fftw_prefix="${withval}"]
		)
	]
)

AC_DEFUN([AC_PREPARE_FFTW],
	[
		AC_REQUIRE([AC_ARG_FFTW])
		AC_REQUIRE([AC_PREPARE_MPI])
		AC_REQUIRE([AX_PTHREAD])
		
		AC_MSG_CHECKING([if the location of FFTW has been given])
		if test -z "${ac_cv_use_fftw_prefix}" ; then
			AC_MSG_RESULT([no])
		else
			AC_MSG_RESULT([${ac_cv_use_fftw_prefix}])
			FFTW_CPPFLAGS="-I${ac_cv_use_fftw_prefix}/include"
			FFTW_LDFLAGS="-L${ac_cv_use_fftw_prefix}/lib"
		fi
		
		if test ! -z "${FFTW_LDFLAGS}" ; then
			saved_LDFLAGS="${LDFLAGS}"
			LDFLAGS="${LDFLAGS} ${FFTW_LDFLAGS}"
		fi
		
		FFTW_MODES=""
		AC_CHECK_LIB([fftw3], [fftw_execute_dft],
			[
				ac_cv_have_fftw=yes
				FFTW_MODES="fftw3"
			]
		)
		AC_CHECK_LIB([fftw3_threads], [fftw_plan_with_nthreads],
			[
				ac_cv_have_fftw_threads=yes
				FFTW_MODES="${FFTW_MODES} fftw3_threads"
			],
			[
			],
			[-lfftw3 ${PTHREAD_LIBS} -lm]
		)
		AC_CHECK_LIB([fftw3_mpi], [fftw_mpi_plan_dft_1d],
			[
				ac_cv_have_fftw_mpi=yes
				FFTW_MODES="${FFTW_MODES} fftw3_mpi"
			],
			[
			],
			[-lfftw3 -lm ${MPI_CLDFLAGS}]
		)
		
		if test ! -z "${FFTW_LDFLAGS}" ; then
			LDFLAGS="${saved_LDFLAGS}"
			unset saved_LDFLAGS
		fi
		
		if test ! -z "${ac_cv_use_fftw_prefix}" -a "${ac_cv_have_fftw}" != "yes" ; then
			AC_MSG_ERROR([FFTW installation location specified but could not be used.])
		fi
		
		AM_CONDITIONAL([HAVE_FFTW], [test "${ac_cv_have_fftw}" = "yes"])
		AM_CONDITIONAL([HAVE_FFTWTHREADS], [test "${ac_cv_have_fftw_threads}" = "yes"])
		AM_CONDITIONAL([HAVE_FFTWMPI], [test "${ac_cv_have_fftw_mpi}" = "yes"])
		
		AC_SUBST([FFTW_CPPFLAGS])
		AC_SUBST([FFTW_LDFLAGS])
		AC_SUBST([FFTW_MODES])
	]
)

AC_DEFUN([AC_DEMAND_FFTW],
	[
		AC_REQUIRE([AC_PREPARE_FFTW])
		
		if test "${ac_cv_have_fftw}" != "yes" ; then
			AC_MSG_ERROR([Could not find the FFTW library.])
		fi
		
	]
)

AC_DEFUN([AC_DEMAND_FFTW_THREADS],
	[
		AC_REQUIRE([AC_DEMAND_FFTW])
		
		if test "${ac_cv_have_fftw_threads}" != "yes" ; then
			AC_MSG_ERROR([Could not find the threaded FFTW library.])
		fi
	]
)

AC_DEFUN([AC_DEMAND_FFTW_MPI],
	[
		AC_REQUIRE([AC_DEMAND_FFTW])
		AC_REQUIRE([AC_DEMAND_MPI])
		
		if test "${ac_cv_have_fftw_mpi}" != "yes" ; then
			AC_MSG_ERROR([Could not find the MPI FFTW library.])
		fi
	]
)


