AC_DEFUN([AC_ARG_MCC],
	[
		AC_ARG_VAR(MCC, [The OmpSs C compiler])
		AC_ARG_VAR(MCFLAGS, [The OmpSs C compiler flags])
	]
)

AC_DEFUN([AC_ARG_MCXX],
	[
		AC_ARG_VAR(MCXX, [The OmpSs C++ compiler])
		AC_ARG_VAR(MCXXFLAGS, [The OmpSs C++ compiler flags])
	]
)

AC_DEFUN([AC_ARG_OMPCC],
	[
		AC_ARG_VAR(OMPCC, [The OpenMP C compiler])
		AC_ARG_VAR(OMPCFLAGS, [The OpenMP C compiler flags])
	]
)

AC_DEFUN([AC_ARG_OMPCXX],
	[
		AC_ARG_VAR(OMPCXX, [The OpenMP C++ compiler])
		AC_ARG_VAR(OMPCXXFLAGS, [The OpenMP C++ compiler flags])
	]
)

AC_DEFUN([AC_BLACKLIST_MCC_VERSION],
	[
		mcc_version=$(${CC} --version | awk '{ print $''2; exit; }')
		AC_MSG_CHECKING([if Mercurium is version $1])
		if test "${mcc_version}" = "$1" ; then
			AC_MSG_RESULT([yes])
			AC_MSG_ERROR([The Mercurium installation is know to be buggy. Please use a newer one.])
		else
			AC_MSG_RESULT([no, good])
		fi
	]
)

AC_DEFUN([AC_BLACKLIST_MCC_GIT_REVISION],
	[
		mcc_git_revision=$(${CC} --version | awk '{ print $''4; exit; }')
		AC_MSG_CHECKING([if Mercurium is git revision $1])
		if test "${mcc_git_revision}" = "$1" ; then
			AC_MSG_RESULT([yes])
			AC_MSG_ERROR([The Mercurium installation is know to be buggy. Please use a newer one.])
		else
			AC_MSG_RESULT([no, good])
		fi
	]
)

AC_DEFUN([AC_MCC_BLACKLIST],
	[
		AC_BLACKLIST_MCC_VERSION([1.99.6])
		AC_BLACKLIST_MCC_GIT_REVISION([46c5867])
	]
)


AC_DEFUN([AC_PREPARE_OMPSS],
	[
		AC_REQUIRE([AC_ARG_MCC])
		AC_REQUIRE([AC_ARG_MCXX])
		
		if test -z "${MCC}" ; then
			MCC=smpcc
		fi
		if test -z "${MCXX}" ; then
			MCXX=smpcxx
		fi
		if test ! -z "${MCC}" ; then
			CC="${MCC} --ompss"
		fi
		if test ! -z "${MCXX}" ; then
			CXX="${MCXX} --ompss"
		fi
		if test ! -z "${MCFLAGS}" ; then
			CFLAGS="${MCFLAGS}"
		fi
		if test ! -z "${MCXXFLAGS}" ; then
			CXXFLAGS="${MCXXFLAGS}"
		fi
		if test "${enable_debug}" = "yes" ; then
			CC="${CC} --debug"
			CXX="${CXX} --debug"
		fi
		if test "${enable_instrument}" = "yes" ; then
			CC="${CC} --instrument"
			CXX="${CXX} --instrument"
		fi
		
		AC_MCC_BLACKLIST
		
		MCC_VERSION_STRING=$(${CC} --version | head -1)
		AC_SUBST(MCC_VERSION_STRING)
		
		NANOX_VERSION_STRING=$(nanox --version | head -1)
		AC_SUBST(NANOX_VERSION_STRING)
	]
)

AC_DEFUN([AC_PREPARE_OPENMP],
	[
		AC_REQUIRE([AC_ARG_OMPCC])
		AC_REQUIRE([AC_ARG_OMPCXX])
		
		if test ! -z "${OMPCC}" ; then
			CC="${OMPCC}"
		fi
		if test ! -z "${OMPCXX}" ; then
			CXX="${OMPCXX}"
		fi
		
		if test ! -z "${OMPCFLAGS}" ; then
			CFLAGS="${OMPCFLAGS}"
		fi
		if test ! -z "${OMPCXXFLAGS}" ; then
			CFLAGS="${OMPCXXFLAGS}"
		fi
		if test ! -z "${CC}" ; then
			if ${CC} --version | grep -q ^mcxx ; then
				# The OpenMP compiler is actually Mercurium
				if test "${enable_debug}" = "yes" ; then
					CC="${CC} --debug"
				fi
				if test "${enable_instrument}" = "yes" ; then
					CC="${CC} --instrument"
				fi
			fi
		fi
		if test ! -z "${CXX}" ; then
			if ${CXX} --version | grep -q ^mcxx ; then
				# The OpenMP compiler is actually Mercurium
				if test "${enable_debug}" = "yes" ; then
					CXX="${CXX} --debug"
				fi
				if test "${enable_instrument}" = "yes" ; then
					CXX="${CXX} --instrument"
				fi
			elif test "${enable_instrument}" = "yes" ; then
				AC_MSG_WARN([Warning: instrumentation not available with current OpenMP compiler.])
			fi
		fi
		
		AC_LANG_PUSH(C)
		AC_OPENMP
		AC_LANG_POP(C)
		
		AC_LANG_PUSH(C++)
		AC_OPENMP
		AC_LANG_POP(C++)
	]
)

AC_DEFUN([_AC_DEMAND_OPENMP],
	[
		AC_REQUIRE([AC_PREPARE_OPENMP])
		# The following uses a program from the original AC_OPENMP autoconf macro
		ac_save_[]_AC_LANG_PREFIX[]FLAGS="$[]_AC_LANG_PREFIX[]FLAGS"
		_AC_LANG_PREFIX[]FLAGS="$[]_AC_LANG_PREFIX[]FLAGS $OPENMP_[]_AC_LANG_PREFIX[]FLAGS"
		
		AC_MSG_CHECKING([if $[]_AC_CC[] $[]_AC_LANG_PREFIX[]FLAGS supports OpenMP])
		
		AC_LINK_IFELSE(
			[_AC_LANG_OPENMP],
			[AC_MSG_RESULT([yes])],
			[
				AC_MSG_RESULT([no])
				AC_MSG_ERROR([the $[]_AC_CC[] compiler does not accept OpenMP. Please set the _AC_CC[]OMP environment variable and if necessary also OMP[]_AC_LANG_PREFIX[]FLAGS.])
			]
		)
		
		_AC_LANG_PREFIX[]FLAGS="$ac_save_[]_AC_LANG_PREFIX[]FLAGS"
	]
)


AC_DEFUN([AC_DEMAND_OPENMP],
	[
		AC_REQUIRE([AC_PREPARE_OPENMP])
		
		AC_LANG_PUSH(C)
		_AC_DEMAND_OPENMP
		AC_LANG_POP(C)
		
		AC_LANG_PUSH(C++)
		_AC_DEMAND_OPENMP
		AC_LANG_POP(C++)
		
	]
)


AC_DEFUN([AC_CHECK_FORTRAN_INTEGER_SIZE],
	[
		AC_REQUIRE([AC_PROG_FC])
		AC_MSG_CHECKING([the Fortran integer type])
		cat > sizetest.f90 << EOF
BLOCK DATA SETUP
INTEGER A,B,C,D
COMMON  /AREA1/ N0,A,B,N1,A2,B2,N2,N22,C,D,N3,C2,D2,N4
DATA    N0,A,B,N1,A2,B2,N2/0,1229074995,1600082772,0,1600082772,1229074995,0/
DATA    N22,C,D,N3,C2,D2,N4/0,858931785,1414750047,0,1414750047,858931785,0/
END
EOF
		if ! ${FC} -c sizetest.f90 ; then
			rm -f sizetest.f90 sizetest.${OBJEXT}
			AC_MSG_ERROR([failed to compile the test code])
		fi
		if grep -q 32BITS__ sizetest.${OBJEXT} ; then
			ac_fortran_integer_size=32
			ac_fortran_integer_type=int32_t
		else
			ac_fortran_integer_size=64
			ac_fortran_integer_type=int64_t
		fi
		rm -f sizetest.f90 sizetest.${OBJEXT}
		AC_MSG_RESULT([${ac_fortran_integer_type}])
	]
)


