AC_DEFUN([AC_ARG_MKL],
	[
		AC_ARG_VAR(MKLROOT, [The location of the MKL library (prefix).])
		AC_ARG_VAR(MKLMODE, [Either ILP64 or LP64.])
	]
)

AC_DEFUN([AC_PREPARE_SEQ_MKL],
	[
		AC_REQUIRE([AC_ARG_MKL])
		
		if test ! -z "${MKLROOT}" ; then
			ac_cv_lapack_type=MKL
			if test -z "${MKLMODE}" ; then
				MKLMODE=ILP64
			fi
			
			case "${MKLMODE}" in
			ILP64)
				MKL_LOWERCASE_MODE=ilp64
				LAPACK_INTEGER_TYPE=int64_t
				;;
			LP64)
				MKL_LOWERCASE_MODE=lp64
				LAPACK_INTEGER_TYPE=int32_t
				;;
			*)
				AC_MSG_ERROR([Invalid value for MKLMODE])
				;;
			esac
			
			AC_REQUIRE([AC_CANONICAL_TARGET])
			case "${target_cpu}" in
			x86_64)
				MKL_ARCH=intel64
				;;
			i386|i486|i586|i686|x86)
				MKL_ARCH=ia32
				;;
			*)
				AC_MSG_WARN([Unknown target architecture. Assuming MIC])
				MKL_ARCH=mic
				;;
			esac
			
			AC_F77_FUNC(dgemm)
			
			saved_LIBS="${LIBS}"
			AC_LANG_PUSH(C)
			# Attempt to detect if we are compiling with ICC
			AC_CHECK_LIB(iomp5, omp_get_num_threads,
				[
					# Using ICC
					ac_cc_is_icc="yes"
				], [
					ac_cc_is_icc="no"
				]
			)
			
			# Attempt to find out the sequential libraries
			if test "${ac_cc_is_icc}" = "yes" ; then
				ac_candidate_seq_mkl_libs="-L${MKLROOT}/lib/${MKL_ARCH} -lmkl_intel_${MKL_LOWERCASE_MODE} -lmkl_core -lmkl_sequential -lpthread -lm"
			else
				ac_candidate_seq_mkl_libs="-Wl,--no-as-needed -L${MKLROOT}/lib/${MKL_ARCH} -lmkl_intel_${MKL_LOWERCASE_MODE} -lmkl_core -lmkl_sequential -lpthread -lm"
			fi
			LIBS="${LIBS} ${ac_candidate_seq_mkl_libs}"
			AC_TRY_LINK_FUNC(${dgemm},
				[
					SEQUENTIAL_LAPACK_LIBS="${ac_candidate_seq_mkl_libs}"
				], [
					AC_MSG_ERROR([Cannot find the correct parallel MKL libraries. Please set THREADED_LAPACK_LIBS instead. Help is available at https://software.intel.com/en-us/articles/intel-mkl-link-line-advisor/])
				]
			)
			LIBS="${saved_LIBS}"
			
			AC_LANG_POP(C)
		fi
	]
)

AC_DEFUN([AC_PREPARE_PAR_MKL],
	[
		AC_REQUIRE([AC_ARG_MKL])
		
		if test ! -z "${MKLROOT}" ; then
			ac_cv_lapack_type=MKL
			if test -z "${MKLMODE}" ; then
				MKLMODE=ILP64
			fi
			
			case "${MKLMODE}" in
			ILP64)
				MKL_LOWERCASE_MODE=ilp64
				LAPACK_INTEGER_TYPE=int64_t
				;;
			LP64)
				MKL_LOWERCASE_MODE=lp64
				LAPACK_INTEGER_TYPE=int64_t
				;;
			*)
				AC_MSG_ERROR([Invalid value for MKLMODE])
				;;
			esac
			
			AC_REQUIRE([AC_CANONICAL_TARGET])
			case "${target_cpu}" in
			x86_64)
				MKL_ARCH=intel64
				;;
			i386|i486|i586|i686|x86)
				MKL_ARCH=ia32
				;;
			*)
				AC_MSG_WARN([Unknown target architecture. Assuming MIC])
				MKL_ARCH=mic
				;;
			esac
			
			AC_F77_FUNC(dgemm)
			
			saved_LIBS="${LIBS}"
			AC_LANG_PUSH(C)
			# Attempt to detect if we are compiling with ICC
			AC_CHECK_LIB(iomp5, omp_get_num_threads,
				[
					# Using ICC
					ac_cc_is_icc="yes"
				], [
					ac_cc_is_icc="no"
				]
			)
			
			# The parallel version of MKL uses OpenMP
			AC_OPENMP
			
			# Attempt to find out the parallel libraries
			if test "${ac_cc_is_icc}" = "yes" ; then
				ac_candidate_par_mkl_libs="-L${MKLROOT}/lib/${MKL_ARCH} -lmkl_intel_${MKL_LOWERCASE_MODE} -lmkl_core -lmkl_intel_thread -liomp5 -lpthread -lm"
			else
				if test -z "${OPENMP_CFLAGS}" ; then
					AC_MSG_WARN([The C compiler may not be using the GCC OpenMP library])
				fi
				ac_candidate_par_mkl_libs="-Wl,--no-as-needed -L${MKLROOT}/lib/${MKL_ARCH} -lmkl_intel_${MKL_LOWERCASE_MODE} -lmkl_core -lmkl_gnu_thread ${OPENMP_CFLAGS} -ldl -lpthread -lm"
			fi
			LIBS="${LIBS} ${ac_candidate_par_mkl_libs}"
			
			AC_TRY_LINK_FUNC(${dgemm},
				[
					THREADED_LAPACK_LIBS="${ac_candidate_par_mkl_libs}"
				], [
					AC_MSG_ERROR([Cannot find the correct parallel MKL libraries. Please set THREADED_LAPACK_LIBS instead. Help is available at https://software.intel.com/en-us/articles/intel-mkl-link-line-advisor/])
				]
			)
			LIBS="${saved_LIBS}"
			
			AC_LANG_POP(C)
		fi
	]
)

AC_DEFUN([AC_USE_SEQ_MKL],
	[
		AC_REQUIRE([AC_PREPARE_SEQ_MKL])
	]
)


AC_DEFUN([AC_USE_PAR_MKL],
	[
		AC_REQUIRE([AC_PREPARE_PAR_MKL])
	]
)


AC_DEFUN([AC_ARG_GENERIC_LAPACK],
	[
		AC_ARG_VAR(SEQUENTIAL_LAPACK_LIBS, [Parameters to link with LAPACK and BLAS in sequential mode.])
		AC_ARG_VAR(THREADED_LAPACK_LIBS, [Parameters to link with LAPACK and BLAS in parallel mode.])
		AC_ARG_VAR(LAPACK_INTEGER_TYPE, [Type of the integers used in the LAPACK/BLAS functions.])
	]
)

AC_DEFUN([AC_PREPARE_GENERIC_LAPACK],
	[
		AC_REQUIRE([AC_ARG_GENERIC_LAPACK])
		if test -z "${ac_cv_lapack_type}" ; then
			AC_F77_FUNC(dgemm)
			
			if test ! -z "${SEQUENTIAL_LAPACK_LIBS}" ; then
				saved_LIBS="${LIBS}"
				LIBS="${LIBS} ${SEQUENTIAL_LAPACK_LIBS}"
				AC_TRY_LINK_FUNC(${dgemm},
					[],
					[
						AC_MSG_ERROR([Invalid value for SEQUENTIAL_LAPACK_LIBS])
					]
				)
				LIBS="${saved_LIBS}"
			else
				for lib in openblas essl scs lapack_atlas lapack ; do
					AC_CHECK_LIB($lib, $dgemm,
						[
							SEQUENTIAL_LAPACK_LIBS="-l${lib}"
						],
						[]
					)
					if test ! -z "${SEQUENTIAL_LAPACK_LIBS}" ; then
						break
					fi
				done
			fi
			
			if test ! -z "${THREADED_LAPACK_LIBS}" ; then
				saved_LIBS="${LIBS}"
				LIBS="${LIBS} ${THREADED_LAPACK_LIBS}"
				AC_TRY_LINK_FUNC(${dgemm},
					[],
					[
						AC_MSG_ERROR([Invalid value for THREADED_LAPACK_LIBS])
					]
				)
				LIBS="${saved_LIBS}"
			else
				for lib in openblas essl scs lapack_atlas lapack ; do
					AC_CHECK_LIB($lib, $dgemm,
						[
							THREADED_LAPACK_LIBS="-l${lib}"
						],
						[]
					)
					if test ! -z "${THREADED_LAPACK_LIBS}" ; then
						break
					fi
				done
			fi
			
			if test ! -z "${SEQUENTIAL_LAPACK_LIBS}" -a ! -z "${THREADED_LAPACK_LIBS}" ; then
				ac_cv_lapack_type="generic"
			fi
			
			if test -z "${LAPACK_INTEGER_TYPE}" ; then
				AC_MSG_WARN([Assuming the LAPACK library uses the default Fortran integer size])
				AC_CHECK_FORTRAN_INTEGER_SIZE
				LAPACK_INTEGER_TYPE=${ac_fortran_integer_type}
				LAPACK_INTEGER_TYPE_GUESSED="(Guessed. Warning: can cause segfaults if incorrect.)"
				AC_SUBST([LAPACK_INTEGER_TYPE_GUESSED])
			fi
		fi
	]
)

AC_DEFUN([AC_PREPARE_SEQ_LAPACK],
	[
		AC_PREPARE_SEQ_MKL
		AC_PREPARE_GENERIC_LAPACK
		AC_F77_WRAPPERS
		
		AM_CONDITIONAL([HAVE_LAPACK], [test ! -z "${SEQUENTIAL_LAPACK_LIBS}"])
		
		AC_LANG_PUSH(C)
		AC_CHECK_LIB([m], [sqrt], [SEQUENTIAL_LAPACK_LIBS="${SEQUENTIAL_LAPACK_LIBS} -lm"])
		AC_LANG_POP(C)
		
		AC_SUBST(SEQUENTIAL_LAPACK_LIBS)
		AC_SUBST(LAPACK_INTEGER_TYPE)
	]
)

AC_DEFUN([AC_PREPARE_PAR_LAPACK],
	[
		AC_PREPARE_PAR_MKL
		AC_PREPARE_GENERIC_LAPACK
		AC_F77_WRAPPERS
		
		AM_CONDITIONAL([HAVE_LAPACK], [test ! -z "${THREADED_LAPACK_LIBS}"])
		
		AC_LANG_PUSH(C)
		AC_CHECK_LIB([m], [sqrt], [THREADED_LAPACK_LIBS="${THREADED_LAPACK_LIBS} -lm"])
		AC_LANG_POP(C)
		
		AC_SUBST(THREADED_LAPACK_LIBS)
		AC_SUBST(LAPACK_INTEGER_TYPE)
	]
)

AC_DEFUN([AC_PREPARE_LAPACK],
	[
		AC_PREPARE_SEQ_MKL
		AC_PREPARE_PAR_MKL
		AC_PREPARE_GENERIC_LAPACK
		
		AM_CONDITIONAL([HAVE_LAPACK], [test ! -z "${SEQUENTIAL_LAPACK_LIBS}" -a ! -z "${THREADED_LAPACK_LIBS}"])
		
		AC_SUBST(SEQUENTIAL_LAPACK_LIBS)
		AC_SUBST(THREADED_LAPACK_LIBS)
		AC_SUBST(LAPACK_INTEGER_TYPE)
	]
)

AC_DEFUN([AC_DEMAND_SEQ_LAPACK],
	[
		AC_REQUIRE([AC_PREPARE_SEQ_LAPACK])
		AC_REQUIRE([AC_PREPARE_SEQ_MKL])
		AC_REQUIRE([AC_PREPARE_GENERIC_LAPACK])
		
		if test -z "${SEQUENTIAL_LAPACK_LIBS}" ; then
			AC_MSG_ERROR([Could not find the LAPACK library. Please try setting the SEQUENTIAL_LAPACK_LIBS environment variable, or MKLROOT and MKLMODE.])
		fi
	]
)

AC_DEFUN([AC_DEMAND_PAR_LAPACK],
	[
		AC_REQUIRE([AC_PREPARE_PAR_LAPACK])
		AC_REQUIRE([AC_PREPARE_PAR_MKL])
		AC_REQUIRE([AC_PREPARE_GENERIC_LAPACK])
		
		if test -z "${THREADED_LAPACK_LIBS}" ; then
			AC_MSG_ERROR([Could not find the LAPACK library. Please try setting the THREADED_LAPACK_LIBS environment variable, or MKLROOT and MKLMODE.])
		fi
	]
)

AC_DEFUN([AC_DEMAND_LAPACK],
	[
		AC_REQUIRE([AC_PREPARE_LAPACK])
		AC_REQUIRE([AC_DEMAND_SEQ_MKL])
		AC_REQUIRE([AC_DEMAND_PAR_MKL])
		
		if test -z "${SEQUENTIAL_LAPACK_LIBS}" -o -z "${THREADED_LAPACK_LIBS}" ; then
			AC_MSG_ERROR([Could not find the LAPACK libraries. Please try setting either the SEQUENTIAL_LAPACK_LIBS and THREADED_LAPACK_LIBS environment variables, or MKLROOT and MKLMODE.])
		fi
	]
)


