#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdint.h>

#include <complex.h>
#ifdef I
#undef I
#endif

#include <fftw3-mpi.h>


#include <mpi.h>
#include "../../utils/timer.h"


#define WISDOM_FILE \
	(getenv("FFTW_WISDOM_FILE") != 0 ? getenv("FFTW_WISDOM_FILE") : "fftw_mpi.wisdom")


fftw_plan plan;


int main(int argc, char **argv) {
	double _Complex *A;
	
	if (argc != 7) {
		fprintf(stderr, "Usage: %s <N> <N> <ignored> <ignored> <MPI processes> <ignored>\n", argv[0]);
		return 1;
	}
	
	long N_SQRT = atol(argv[1]);
	int nprocs = atoi(argv[5]);
	
	long N = N_SQRT*N_SQRT;
	
	//MPI_Init(&argc, &argv);
	MPI_Init(NULL, NULL);
	
	fftw_mpi_init();
	
	ptrdiff_t local_ni, local_i_start, local_no, local_o_start;
	ptrdiff_t alloc_local =
		fftw_mpi_local_size_1d(N, MPI_COMM_WORLD, FFTW_FORWARD, FFTW_PATIENT, &local_ni, &local_i_start, &local_no, &local_o_start);
	
	if ((local_ni != local_no) || (local_i_start != local_o_start)) {
		fprintf(stderr, "Error: unexpected transform size.\n");
		MPI_Abort(MPI_COMM_WORLD, 1);
		exit(1);
	}
	
	A = (double _Complex *) fftw_malloc( sizeof(double _Complex) * local_ni );
	if (A == NULL) {
		fprintf(stderr, "Error: memory allocation failed.\n");
		MPI_Abort(MPI_COMM_WORLD, 2);
		exit(2);
	}
	
	// Pre-fault the memory
	memset(A, 0, local_ni * sizeof(double _Complex));
	
	int rank;
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	
	// Load the wisdom
	if (rank == 0) {
		FILE *file = fopen(WISDOM_FILE, "r");
		if (file != NULL) {
			fftw_import_wisdom_from_file(file);
			fclose(file);
		}
	}
	fftw_mpi_broadcast_wisdom(MPI_COMM_WORLD);
	
	fftw_plan plan = fftw_mpi_plan_dft_1d(N, A, A, MPI_COMM_WORLD, FFTW_FORWARD, FFTW_PATIENT);
	
	// Initialize the data
	for (long i=0; i<local_ni; i++) {
		A[i] = (double)(local_i_start + i) + _Complex_I * (local_i_start - i);
	}
	
	// Execute
	START_TIME();
	fftw_execute(plan);
	STOP_TIME();
	
	fftw_free(A);
	
	if (rank == 0) {
		if (getenv("MEASURE_TIME") == NULL) {
			printf("%f", (5.0*(double)N*log(N)/log(2.0))/GET_TIME());
		} else {
			printf("%f", GET_TIME() / 1000000.0);
		}
	}
	
	// Save the wisdom
	fftw_mpi_gather_wisdom(MPI_COMM_WORLD);
	if (rank == 0) {
		FILE *file = fopen(WISDOM_FILE, "w");
		fftw_export_wisdom_to_file(file);
		fclose(file);
	}
	
	fftw_destroy_plan(plan);
	MPI_Finalize();
	
	return 0;
}

