#include <mpi.h>
#include "../../utils/timer.h"


static void triad(long size, double a[restrict size], double b[restrict size], double c[restrict size], double scalar) {
	for (long i = 0; i < size; i++)
		a[i] = b[i] + scalar*c[i];
}


int main(int argc, char **argv) {
	if (argc != 7) {
		fprintf(stderr, "Usage: %s <N> <N> <ignored> <ignored> <MPI processes> <ignored>\n", argv[0]);
		return 1;
	}
	
	long globalN = atol(argv[1]) * 1024L * 1024L;
	int nprocs = atoi(argv[5]);
	
	MPI_Init(&argc, &argv);
// 	MPI_Init(NULL, NULL);
	
	long N = globalN / nprocs;
	
	double *a = (double *)malloc(sizeof(double) * N);
	double *b = (double *)malloc(sizeof(double) * N);
	double *c = (double *)malloc(sizeof(double) * N);
	double scalar = 3.0;
	
	// Data initialization
	for (long i = 0; i < N; i++) {
		a[i] = 0.0;
		b[i] = 2.0;
		c[i] = 1.0;
	}
	
	triad(N, a, b, c, scalar);
	MPI_Barrier(MPI_COMM_WORLD);
	
	START_TIME();
	triad(N, a, b, c, scalar);
	MPI_Barrier(MPI_COMM_WORLD);
	STOP_TIME();
	
	int rank;
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	
	if (rank == 0) {
		if (getenv("MEASURE_TIME") == NULL) {
			printf("%f", ((double)(globalN*sizeof(double)*3)) / ((double)GET_TIME()));
		} else {
			printf("%f", GET_TIME() / 1000000.0);
		}
	}
	
	MPI_Finalize();
	
	return 0;
}

