#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>


#include "../../../utils/timer.h"


#define TYPE double
#include "../../../utils/timer.h"
#include "blocks/common/init.h"


static inline double initializeElement(long I, long J, long i, long j) {
	fprintf(stderr, "This function is here only to avoid linking problems when debugging is turned on.\n");
}

static void initialize_segment(long size, double a[restrict size], double b[restrict size], double c[restrict size], double a_scalar, double b_scalar, double c_scalar) {
	for (long i = 0; i < size; i++) {
		a[i] = a_scalar;
		b[i] = b_scalar;
		c[i] = c_scalar;
	}
}


static void triad(long size, double a[restrict size], double b[restrict size], double c[restrict size], double scalar) {
	for (long i = 0; i < size; i++)
		a[i] = b[i] + scalar*c[i];
}


int main(int argc, char ** argv) {
	if (argc != 7) {
		fprintf(stderr, "Usage: %s <N/2^20> <N/2^20> <BS/2^20> <BS/2^20> <threads> <I|B>\n", argv[0]);
		return 1;
	}
	
	long const N = atol(argv[1]) * 1024L * 1024L;
	long const BSIZE = atol(argv[3]) * 1024L * 1024L;
	parseInitializationType(argv[6]);
	
	double *a = (double *)malloc(sizeof(double) * N);
	double *b = (double *)malloc(sizeof(double) * N);
	double *c = (double *)malloc(sizeof(double) * N);
	double scalar = 3.0;
	
	#pragma omp parallel for
	for (long i = 0; i < N; i+= BSIZE) {
		#pragma omp task
		initialize_segment(BSIZE, &a[i], &b[i], &c[i], 0.0, 2.0, 1.0);
	}
	
	#pragma omp parallel for
	for (long i = 0; i < N; i += BSIZE) {
		#pragma omp task untied
		triad(BSIZE, &a[i], &b[i], &c[i], scalar);
	}
	
	START_TIME();
	#pragma omp parallel for
	for (long i = 0; i < N; i += BSIZE) {
		#pragma omp task untied
		triad(BSIZE, &a[i], &b[i], &c[i], scalar);
	}
	STOP_TIME();
	
	if (getenv("MEASURE_TIME") == NULL) {
		printf("%f", ((double)(N*sizeof(double)*3)) / ((double)GET_TIME()));
	} else {
		printf("%f", GET_TIME() / 1000000.0);
	}
	
	return 0;
}
