/*
 * Copyright (c) 2008, BSC (Barcelona Supercomputing Center)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the <organization> nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY BSC ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h> 


#define TYPE double
#include "flat/common/init.h"
#include "../../../utils/timer.h"


static inline double initializeElement(long I, long J, long i, long j) {
	return ((i+I)*(j+J)) % 1325;
}


static double *alloc_and_gen_matrix(long N, long BS) {
	typedef double (*m_t)[N];
	
	m_t A = (m_t) malloc(sizeof(double)*N*N);
	
	if (*A == NULL) {
		fprintf(stderr, "Memory allocation error %iMB\n", (sizeof(double)*N*N)/(1024L*1024L));
		exit(1);
	}
	
	initializeUsingUserSpecification(N, N, BS, BS, A);
	
	return (double *)A;
}



void gauss_seidel(long N, long BS, double A[N][N]);


int main(int argc, char* argv[]) {
	if (argc != 7) {
		fprintf(stderr, "Usage: %s <N> <N> <block size> <block size> <index> <PL|H|V|B|PW|SEQ>\n", argv[0]);
		return 1;
	}
	parseInitializationType(argv[6]);
	
	long N = atol(argv[1]);
	long BS = atol(argv[3]);
	// long L = atol(argv[4]);
	long L = 16;
	
	if (BS > N) {
		printf("0");
		return 1;
	}
	
	typedef double (*m_t)[N + BS*2];
	
	m_t A = (m_t) alloc_and_gen_matrix(N + BS*2, BS);
	
	START_TIME();
	for (int iters=0; iters<L; iters++) {
		for (long a=BS; a <= N; a+=BS) {
			#pragma omp parallel for
			for (long j=BS; j <= a; j+=BS) {
				#pragma omp task untied
				{
					long i = a + BS - j;
					gauss_seidel( N + BS*2, BS, (m_t) &A[i][j] );
				}
			}
		}
		
		for (long a=BS+BS; a <= N; a+=BS) {
			#pragma omp parallel for
			for (long i=a; i <= N; i+=BS) {
				#pragma omp task untied
				{
					long j = N + a - i;
					gauss_seidel( N + BS*2, BS, (m_t) &A[i][j] );
				}
			}
		}
	}
	STOP_TIME();
	
	if (getenv("MEASURE_TIME") == NULL) {
		// Megaelements per second
		printf("%f", ((double)(L*N*N))/GET_TIME());
	} else {
		printf("%f", GET_TIME() / 1000000.0);
	}
	
	return 0;
}

