#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>


#include "../../../utils/timer.h"

#define OVERRIDE_MALLOC_AND_FREE
#include "memory.h"


#pragma omp task out([size]a, [size]b, [size]c)
static void initialize_segment(long size, double a[restrict size], double b[restrict size], double c[restrict size], double a_scalar, double b_scalar, double c_scalar) {
	for (long i = 0; i < size; i++) {
		a[i] = a_scalar;
		b[i] = b_scalar;
		c[i] = c_scalar;
	}
}


#pragma omp task in([size]b, [size]c) out([size]a)
static void triad(long size, double a[restrict size], double b[restrict size], double c[restrict size], double scalar) {
	for (long i = 0; i < size; i++)
		a[i] = b[i] + scalar*c[i];
}


int main(int argc, char ** argv) {
	if (argc != 7) {
		fprintf(stderr, "Usage: %s <N/2^20> <N/2^20> <BS/2^20> <BS/2^20> <threads> <ignored>\n", argv[0]);
		return 1;
	}
	
	long const N = atol(argv[1]) * 1024L * 1024L;
	long const BSIZE = atol(argv[3]) * 1024L * 1024L;
	
	double *a = (double *)malloc(sizeof(double) * N);
	double *b = (double *)malloc(sizeof(double) * N);
	double *c = (double *)malloc(sizeof(double) * N);
	double scalar = 3.0;
	
	for (long i = 0; i < N; i+= BSIZE)
		initialize_segment(BSIZE, &a[i], &b[i], &c[i], 0.0, 2.0, 1.0);
	#pragma omp taskwait
	
	for (long i = 0; i < N; i += BSIZE)
		triad(BSIZE, &a[i], &b[i], &c[i], scalar);
	
	#pragma omp taskwait
	START_TIME();
	for (long i = 0; i < N; i += BSIZE)
		triad(BSIZE, &a[i], &b[i], &c[i], scalar);
	#pragma omp taskwait
	STOP_TIME();
	
	if (getenv("MEASURE_TIME") == NULL) {
		printf("%f", ((double)(N*sizeof(double)*3)) / ((double)GET_TIME()));
	} else {
		printf("%f", GET_TIME() / 1000000.0);
	}
	
	return 0;
}
