#include <math.h>
#include <stdint.h>

#include <complex.h>
#ifdef I
#undef I
#endif


// Micro block size
#ifndef mBS
#define mBS 2
#endif

#ifndef tw_mBS
#define tw_mBS 2
#endif

#ifndef middleBS
#define middleBS 8
#endif


#ifndef M_PI 
#define M_PI 3.14159265358979323846 
#endif

#define SIGMA -1.0
#define SIGMA2PI (SIGMA)*M_PI*2.0

//
// Helper functions
//

static inline __attribute__((always_inline)) void micro_trsp_blk(long N, long N_SQRT, double _Complex (*data)[N_SQRT])
{
#if mBS == 1
#elif mBS == 2
	register double _Complex a00, a01;
	register double _Complex a10, a11;
	
	a00 = data[0][0]; a01 = data[0][1];
	a10 = data[1][0]; a11 = data[1][1];
	
	data[0][0] = a00; data[0][1] = a10;
	data[1][0] = a01; data[1][1] = a11;
#elif mBS == 4
	register double _Complex a00, a01, a02, a03;
	register double _Complex a10, a11, a12, a13;
	register double _Complex a20, a21, a22, a23;
	register double _Complex a30, a31, a32, a33;
	
	a00 = data[0][0]; a01 = data[0][1]; a02 = data[0][2]; a03 = data[0][3];
	a10 = data[1][0]; a11 = data[1][1]; a12 = data[1][2]; a13 = data[1][3];
	a20 = data[2][0]; a21 = data[2][1]; a22 = data[2][2]; a23 = data[2][3];
	a30 = data[3][0]; a31 = data[3][1]; a32 = data[3][2]; a33 = data[3][3];
	
	data[0][0] = a00; data[0][1] = a10; data[0][2] = a20; data[0][3] = a30;
	data[1][0] = a01; data[1][1] = a11; data[1][2] = a21; data[1][3] = a31;
	data[2][0] = a02; data[2][1] = a12; data[2][2] = a22; data[2][3] = a32;
	data[3][0] = a03; data[3][1] = a13; data[3][2] = a23; data[3][3] = a33;
#else
#error Unimplemented micro block size
#endif
}


static inline __attribute__((always_inline)) void micro_trsp_swap(long N, long N_SQRT, double _Complex (*data1)[N_SQRT], double _Complex (*data2)[N_SQRT])
{
#if mBS == 1
	register double _Complex a00 = data1[0][0];
	register double _Complex b00 = data2[0][0];
	data1[0][0] = b00;
	data2[0][0] = a00;
#elif mBS == 2
	register double _Complex a00, a01;
	register double _Complex a10, a11;
	
	register double _Complex b00, b01;
	register double _Complex b10, b11;
	
	a00 = data1[0][0]; a01 = data1[0][1];
	a10 = data1[1][0]; a11 = data1[1][1];
	
	b00 = data2[0][0]; b01 = data2[0][1];
	b10 = data2[1][0]; b11 = data2[1][1];
	
	data1[0][0] = b00; data1[0][1] = b10;
	data1[1][0] = b01; data1[1][1] = b11;
	
	data2[0][0] = a00; data2[0][1] = a10;
	data2[1][0] = a01; data2[1][1] = a11;
#elif mBS == 4
	register double _Complex a00, a01, a02, a03;
	register double _Complex a10, a11, a12, a13;
	register double _Complex a20, a21, a22, a23;
	register double _Complex a30, a31, a32, a33;
	
	register double _Complex b00, b01, b02, b03;
	register double _Complex b10, b11, b12, b13;
	register double _Complex b20, b21, b22, b23;
	register double _Complex b30, b31, b32, b33;
	
	a00 = data1[0][0]; a01 = data1[0][1]; a02 = data1[0][2]; a03 = data1[0][3];
	a10 = data1[1][0]; a11 = data1[1][1]; a12 = data1[1][2]; a13 = data1[1][3];
	a20 = data1[2][0]; a21 = data1[2][1]; a22 = data1[2][2]; a23 = data1[2][3];
	a30 = data1[3][0]; a31 = data1[3][1]; a32 = data1[3][2]; a33 = data1[3][3];
	
	b00 = data2[0][0]; b01 = data2[0][1]; b02 = data2[0][2]; b03 = data2[0][3];
	b10 = data2[1][0]; b11 = data2[1][1]; b12 = data2[1][2]; b13 = data2[1][3];
	b20 = data2[2][0]; b21 = data2[2][1]; b22 = data2[2][2]; b23 = data2[2][3];
	b30 = data2[3][0]; b31 = data2[3][1]; b32 = data2[3][2]; b33 = data2[3][3];
	
	data1[0][0] = b00; data1[0][1] = b10; data1[0][2] = b20; data1[0][3] = b30;
	data1[1][0] = b01; data1[1][1] = b11; data1[1][2] = b21; data1[1][3] = b31;
	data1[2][0] = b02; data1[2][1] = b12; data1[2][2] = b22; data1[2][3] = b32;
	data1[3][0] = b03; data1[3][1] = b13; data1[3][2] = b23; data1[3][3] = b33;
	
	data2[0][0] = a00; data2[0][1] = a10; data2[0][2] = a20; data2[0][3] = a30;
	data2[1][0] = a01; data2[1][1] = a11; data2[1][2] = a21; data2[1][3] = a31;
	data2[2][0] = a02; data2[2][1] = a12; data2[2][2] = a22; data2[2][3] = a32;
	data2[3][0] = a03; data2[3][1] = a13; data2[3][2] = a23; data2[3][3] = a33;
#else
#error Unimplemented micro block size
#endif
}


static inline __attribute__((always_inline)) void micro_tw_trsp_blk(long N, long N_SQRT, long I, double _Complex (*data)[N_SQRT])
{
#if tw_mBS == 1
	register double _Complex w00 = cos(SIGMA2PI*I*I/N) + _Complex_I * sin(SIGMA2PI*I*I/N);
	register double _Complex a00 = data[0][0] * w00;
	data[0][0] = a00;
#elif tw_mBS == 2
	register double _Complex a00, a01;
	register double _Complex a10, a11;
	
	register double _Complex w00;
	register double _Complex w10, w11;
	
	w00 = cexp(_Complex_I * SIGMA2PI*I*I/N);
	w10 = cexp(_Complex_I * SIGMA2PI*(I+1.0)*I/N);
	w11 = cexp(_Complex_I * SIGMA2PI*(I+1.0)*(I+1.0)/N);
	
	a00 = data[0][0] * w00; a01 = data[0][1] * w10;
	a10 = data[1][0] * w10; a11 = data[1][1] * w11;
	
	data[0][0] = a00; data[0][1] = a10;
	data[1][0] = a01; data[1][1] = a11;
#else
#error Unimplemented micro block size
#endif
}


static inline __attribute__((always_inline)) void micro_tw_trsp_swap(long N, long N_SQRT, long I, long J, double _Complex (*data1)[N_SQRT], double _Complex (*data2)[N_SQRT])
{
#if tw_mBS == 1
	register double _Complex w00 = cos(SIGMA2PI*I*J/N) + _Complex_I * sin(SIGMA2PI*I*J/N);
	register double _Complex a00 = data1[0][0] * w00;
	register double _Complex b00 = data2[0][0] * w00;
	data1[0][0] = b00;
	data2[0][0] = a00;
#elif tw_mBS == 2
	register double _Complex a00, a01;
	register double _Complex a10, a11;
	
	register double _Complex b00, b01;
	register double _Complex b10, b11;
	
	register double _Complex w00, w01;
	register double _Complex w10, w11;
	
	w00 = cexp(_Complex_I * SIGMA2PI*I*J/N);
	w01 = cexp(_Complex_I * SIGMA2PI*I*(J+1.0)/N);
	w10 = cexp(_Complex_I * SIGMA2PI*(I+1.0)*J/N);
	w11 = cexp(_Complex_I * SIGMA2PI*(I+1.0)*(J+1.0)/N);
	
	a00 = data1[0][0] * w00; a01 = data1[0][1] * w01;
	a10 = data1[1][0] * w10; a11 = data1[1][1] * w11;
	
	b00 = data2[0][0] * w00; b01 = data2[0][1] * w10;
	b10 = data2[1][0] * w01; b11 = data2[1][1] * w11;
	
	data1[0][0] = b00; data1[0][1] = b10;
	data1[1][0] = b01; data1[1][1] = b11;
	
	data2[0][0] = a00; data2[0][1] = a10;
	data2[1][0] = a01; data2[1][1] = a11;
#else
#error Unimplemented micro block size
#endif
}



//
// Middle level functions
//

static void middle_trsp_blk(long N, long N_SQRT, double _Complex data[N_SQRT][N_SQRT]) {
	typedef double _Complex (*m_p)[N_SQRT];
	
	for (long i=0; i < middleBS; i+=mBS) {
		for (long j=0; j < i; j+=mBS) {
			micro_trsp_swap(N, N_SQRT, (m_p) &data[i][j], (m_p) &data[j][i]);
		}
		micro_trsp_blk(N, N_SQRT, (m_p) &data[i][i]);
	}
}

static void middle_trsp_swap (long N, long N_SQRT, double _Complex data1[N_SQRT][N_SQRT], double _Complex data2[N_SQRT][N_SQRT]) {
	typedef double _Complex (*m_p)[N_SQRT];
	
	for (long i=0; i < middleBS; i+=mBS) {
		for (long j=0; j < middleBS; j+=mBS) {
			micro_trsp_swap(N, N_SQRT, (m_p) &data1[i][j], (m_p) &data2[j][i]);
		}
	}
}


static void middle_tw_trsp_blk(long N, long N_SQRT, long I, double _Complex data[N_SQRT][N_SQRT]) {
	typedef double _Complex (*m_p)[N_SQRT];
	
	for (long i=0; i < middleBS; i+=tw_mBS) {
		for (long j=0; j < i; j+=tw_mBS) {
			micro_tw_trsp_swap(N, N_SQRT, I+i, I+j, (m_p) &data[i][j], (m_p) &data[j][i]);
		}
		micro_tw_trsp_blk(N, N_SQRT, I+i, (m_p) &data[i][i]);
	}
}


static void middle_tw_trsp_swap (long N, long N_SQRT, long I, long J, double _Complex data1[N_SQRT][N_SQRT], double _Complex data2[N_SQRT][N_SQRT]) {
	typedef double _Complex (*m_p)[N_SQRT];
	
	for (long i=0; i < middleBS; i+=tw_mBS) {
		for (long j=0; j < middleBS; j+=tw_mBS) {
			micro_tw_trsp_swap(N, N_SQRT, I+i, J+j, (m_p) &data1[i][j], (m_p) &data2[j][i]);
		}
	}
}




//
// Tasks
//

#pragma omp task inout(([N_SQRT]data)[0;TR_BS][0;TR_BS])
void trsp_blk(long N, long N_SQRT, long TR_BS, double _Complex data[N_SQRT][N_SQRT]) {
	typedef double _Complex (*m_p)[N_SQRT];
	
	for (long i=0; i < TR_BS; i+=middleBS) {
		for (long j=0; j < i; j+=middleBS) {
			middle_trsp_swap(N, N_SQRT, (m_p) &data[i][j], (m_p) &data[j][i]);
		}
		middle_trsp_blk(N, N_SQRT, (m_p) &data[i][i]);
	}
}

#pragma omp task inout(([N_SQRT]data1)[0;TR_BS][0;TR_BS], ([N_SQRT]data2)[0;TR_BS][0;TR_BS])
void trsp_swap (long N, long N_SQRT, long TR_BS, double _Complex data1[N_SQRT][N_SQRT], double _Complex data2[N_SQRT][N_SQRT]) {
	typedef double _Complex (*m_p)[N_SQRT];
	
	for (long i=0; i < TR_BS; i+=middleBS) {
		for (long j=0; j < TR_BS; j+=middleBS) {
			middle_trsp_swap(N, N_SQRT, (m_p) &data1[i][j], (m_p) &data2[j][i]);
		}
	}
}

#pragma omp task inout(([N_SQRT]data)[0;TR_BS][0;TR_BS])
void tw_trsp_blk(long N, long N_SQRT, long TR_BS, long I, double _Complex data[N_SQRT][N_SQRT]) {
	typedef double _Complex (*m_p)[N_SQRT];
	
	for (long i=0; i < TR_BS; i+=middleBS) {
		for (long j=0; j < i; j+=middleBS) {
			middle_tw_trsp_swap(N, N_SQRT, I+i, I+j, (m_p) &data[i][j], (m_p) &data[j][i]);
		}
		middle_tw_trsp_blk(N, N_SQRT, I+i, (m_p) &data[i][i]);
	}
}

#pragma omp task inout(([N_SQRT]data1)[0;TR_BS][0;TR_BS], ([N_SQRT]data2)[0;TR_BS][0;TR_BS])
void tw_trsp_swap (long N, long N_SQRT, long TR_BS, long I, long J, double _Complex data1[N_SQRT][N_SQRT], double _Complex data2[N_SQRT][N_SQRT]) {
	typedef double _Complex (*m_p)[N_SQRT];
	
	for (long i=0; i < TR_BS; i+=middleBS) {
		for (long j=0; j < TR_BS; j+=middleBS) {
			middle_tw_trsp_swap(N, N_SQRT, I+i, J+j, (m_p) &data1[i][j], (m_p) &data2[j][i]);
		}
	}
}


