/*
 * Copyright (c) 2008, BSC (Barcelona Supercomputing Center)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the <organization> nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY BSC ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h> 


#define TYPE double
#include "regions/common/init2.h"
#include "../../../utils/timer.h"


static inline double initializeElement(long I, long J, long i, long j) {
	return ((i+I)*(j+J)) % 1325;
}


static double *alloc_and_gen_matrix(long N, long BS) {
	long realN = N*2L;
	if (N == BS) {
		realN = realN * 2L;
	}
	
	typedef double (*m_t)[realN][realN];
	
	m_t A = (m_t) malloc(sizeof(double)*realN*realN*2L);
	
	if (*A == NULL) {
		fprintf(stderr, "Memory allocation error %iMB\n", (sizeof(double)*N*N*4L*2L)/(1024L*1024L));
		exit(1);
	}
	
	initializeUsingUserSpecification(N, N, realN, realN, BS, BS, &A[0][BS][BS]);
	
	// This barrier should allow the runtime to balance the data distribution of the "interesting" part of the matrix
	#pragma omp taskwait
	
	initializeUsingUserSpecification(BS, N+BS, realN, realN, BS, BS, &A[0][0][0]);
	initializeUsingUserSpecification(BS, N+BS, realN, realN, BS, BS, &A[0][N+BS][0]);
	initializeUsingUserSpecification(N, BS, realN, realN, BS, BS, &A[0][BS][0]);
	initializeUsingUserSpecification(N, BS, realN, realN, BS, BS, &A[0][0][N+BS]);
	
	return (double *)A;
}



#pragma omp task in(([realN]oldPanel)[1;BS][1;BS], ([realN]oldPanel)[0][1;BS], ([realN]oldPanel)[BS+1][1;BS], ([realN]oldPanel)[1;BS][0], ([realN]oldPanel)[1;BS][BS+1]) out(([realN]newPanel)[1;BS][1;BS])
void jacobi(long realN, long BS, double newPanel[realN][realN], double oldPanel[realN][realN]);


int main(int argc, char* argv[]) {
	if (argc != 7) {
		fprintf(stderr, "Usage: %s <N> <N> <block size> <block size> <index> <H|V|B|PW|SEQ>\n", argv[0]);
		return 1;
	}
	parseInitializationType(argv[6]);
	
	long N = atol(argv[1]);
	long BS = atol(argv[3]);
	// long L = atol(argv[4]);
	long L = 16;
	
	if (BS > N) {
		printf("0");
		return 1;
	}
	
	long realN = N*2L;
	if (N == BS) {
		realN = realN * 2L;
	}
	
	typedef double (*m_t)[realN][realN];
	
	m_t A = (m_t) alloc_and_gen_matrix(N, BS);
	
	#pragma omp taskwait
	START_TIME();
	for (int iters=0; iters<L; iters++) {
		int currentPanel = (iters + 1) % 2;
		int lastPanel = iters % 2;
		
		for (long i=BS; i <= N; i+=BS) {
			for (long j=BS; j <= N; j+=BS) {
				jacobi( realN, BS, (m_t) &A[currentPanel][i-1][j-1], (m_t) &A[lastPanel][i-1][j-1] );
			}
		}
	}
	#pragma omp taskwait
	STOP_TIME();
	
	if (getenv("MEASURE_TIME") == NULL) {
		// Megaelements per second
		printf("%f", ((double)(L*N*N))/GET_TIME());
	} else {
		printf("%f", GET_TIME() / 1000000.0);
	}
	
	return 0;
}

