#include <stdio.h>
#include <stdlib.h>


// N and MIN_BS must be powers of 2
extern long N;
extern long MIN_SORT_SIZE;
extern long MIN_MERGE_SIZE;

#define T int


#define INSERTION_SIZE 20

static void insertion_sort(T *low, T *high)
{
	T *p, *q;
	T a, b;
	for (q = low + 1; q <= high; ++q) {
		a = q[0];
		for (p = q - 1; p >= low && (b = p[0]) > a; p--) {
			p[1] = b;
		}
		p[1] = a;
	}
}

static T med3(T a, T b, T c) {
	if (a < b)
		if (b < c)
			return b;
		else
			if (a < c)
				return c;
			else
				return a;
	else
		if (b > c)
			return b;
		else
			if (a > c)
				return c;
			else
				return a;
}

/*
 * simple approach for now; a better median-finding
 * may be preferable
 */
static T choose_pivot(T *low, T *high)
{
	return med3(*low, *high, low[(high-low)/2]);
}

static T *seqpart(T *low, T *high) {
	T pivot;
	T h, l;
	T *curr_low = low;
	T *curr_high = high;

	pivot = choose_pivot(low, high);

	while (1) {
		while ((h = *curr_high) > pivot)
			curr_high--;

		while ((l = *curr_low) < pivot)
			curr_low++;

		if (curr_low >= curr_high)
			break;

		*curr_high-- = l;
		*curr_low++ = h;
	}

	/*
	* I don't know if this is really necessary.
	* The problem is that the pivot is not always the
	* first element, and the partition may be trivial.
	* However, if the partition is trivial, then
	* *high is the largest element, whence the following
	* code.
	*/
	if (curr_high < high)
		return curr_high;
	else
		return curr_high - 1;
}

static void seqquick(T *low, T *high) {
	while (high - low > INSERTION_SIZE) {
		T *p = seqpart(low, high);
		seqquick(low, p);
		low = p + 1;
	}
	
	insertion_sort(low, high);
}

void basicsort(long n, T data[n]) {
	seqquick(data, data + n - 1);
}


static inline int pivots_are_aligned(T *left, T *right, long n, long leftStart, long rightStart) {
	if (leftStart == 0 || rightStart == 0 || leftStart == n || rightStart == n) {
		return 1;
	}
	
	if (left[leftStart] <= right[rightStart] && right[rightStart-1] <= left[leftStart]) {
		return 1;
	}
	if (right[rightStart] <= left[leftStart] && left[leftStart-1] <= right[rightStart]) {
		return 1;
	}
	
	return 0;
}


static inline int must_decrease_left(T *left, T *right, long n, long leftStart, long rightStart) {
	return (left[leftStart] > right[rightStart]);
}


static inline long min(long a, long b) {
	if (a < b) {
		return a;
	} else {
		return b;
	}
}


static inline void find_pivot(T *left, T *right, long n, long start, long *leftStart, long *rightStart) {
	*leftStart = start/2L;
	*rightStart = start/2L;
	
	if (start == 0) {
		return;
	}
	
	long jumpSize;
	if (pivots_are_aligned(left, right, n, *leftStart, *rightStart)) {
		return;
	} else if (must_decrease_left(left, right, n, *leftStart, *rightStart)) {
		jumpSize = min(start/2L, n - start/2L) / 2L;
		*leftStart -= jumpSize;
		*rightStart += jumpSize;
	} else {
		jumpSize = min(start/2L, n - start/2L) / 2L;
		*leftStart += jumpSize;
		*rightStart -= jumpSize;
	}
	
	while (1) {
		if (pivots_are_aligned(left, right, n, *leftStart, *rightStart)) {
			return;
		} else if (must_decrease_left(left, right, n, *leftStart, *rightStart)) {
			jumpSize = (jumpSize+1L)/2L; // At least jump by 1
			*leftStart -= jumpSize;
			*rightStart += jumpSize;
		} else {
			jumpSize = (jumpSize+1L)/2L; // At least jump by 1
			*leftStart += jumpSize;
			*rightStart -= jumpSize;
		}
	}
}


void merge_task(long n, T left[n], T right[n], T result[n*2], long start, long length) {
	long leftStart, rightStart;
	find_pivot(left, right, n, start, &leftStart, &rightStart);
	//printf("Merge %x[%i:%i] %x[%i:%i] -> %x[%i:%i]\n\n", left, leftStart, n, right, rightStart, n, result, start, length);
	
	result += start;
	while (length != 0) {
		if (leftStart == n) {
			*result = right[rightStart];
			rightStart++;
			result++;
		} else if (rightStart == n) {
			*result = left[leftStart];
			leftStart++;
			result++;
		} else if (left[leftStart] <= right[rightStart]) {
			*result = left[leftStart];
			leftStart++;
			result++;
		} else {
			*result = right[rightStart];
			rightStart++;
			result++;
		}
		length--;
	}
}


