#!/bin/sh

if [ "${V}" = "1" ] ; then
	set -x
fi


usage() {
	cat << EOF
Usage: $0 <NP> [S|M|L|--help]
	NP:	Number of threads/processes
	S:	Run small benchmarks (default)
	M:	Run medium benchmarks
	L:	Run large benchmarks
	
	--help:	This help message
	
	MPIRUN_PARAMS can be set to any extra parameters that should be passed to mpirun.
	NX_INSTRUMENTATION can be set to "extrae" to execute the benchmarks with tracing.
EOF
}

basedir=$(dirname $0)
scriptdir=${basedir}/../scripts

if [ "$#" -eq 0 ] ; then
	usage
	exit 1
fi

export NP=$1
shift

if [ "$#" -eq 0 ] ; then
	# By default run the benchmarks with the S size
	params=S
else
	params="$*"
fi


# Verify the parameters
for param in ${params} ; do
	case "${param}" in
	"--help")
		usage
		exit
		;;
	S|M|L)
		;;
	*)
		echo Error: invalid parameter \"${param}\".
		usage
		exit 1
		;;
	esac
done




passed=0
failed=0
for bench in $(${scriptdir}/detect_benchmarks "${basedir}") ; do
	echo '*** '${bench}' ***'
	src_dir_entry="${basedir}/${bench}"
	binary=$(echo ${bench} | awk -F / '{ res=""; for (i=NF; i>1; i--) res = res $i "_"; print res $1; }')
	build_dir_entry="${bench}"
	executable="${build_dir_entry}/${binary}"
	algorithm=$(basename "${bench}")
	
	if [ ! -z "${SKIP_MPI}" ] ; then
		case "${binary}" in
			*_mpi)
				echo 'Skipping MPI executable'
				continue
			;;
		esac
	fi
	
	# Implementation-specific parameters
	sizer_script="${src_dir_entry}/get_parameters_for_size"
	if [ ! -x "${sizer_script}" ] ; then
		# Or parameters common to all the implementations of the benchmark
		sizer_script="${basedir}/common/${algorithm}/get_parameters_for_size"
	fi
	
	if [ -x "${executable}" -a -x "${sizer_script}" ] ; then
		for problem_size in ${params} ; do
			result=$(${basedir}/run-one.sh ${NP} ${problem_size} ${bench} ${binary})
			rc=$?
			
			if [ "$rc" != "0" -o "${result}" = "0" -o -z "${result}" ] ; then
				failed=$((${failed} + 1))
				echo "	FAILED"
			else
				passed=$((${passed} + 1))
				echo "	Result: ${result}"
			fi
		done
	fi
	echo
	echo
done


echo '*** 'SUMMARY' ***'
echo "Successful:	${passed}"
echo "Failed:	${failed}"

if [ "${failed}" -gt 0 ] ; then
	exit 1
fi


