#ifndef integer
#error This benchmark requires integer to be defined to the equivalent Fortran integer type
#endif

#include <errno.h>
#include <float.h>
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>

#define TYPE double
#include "../../utils/blas.h"
#include "../../utils/blas_init.h"
#include "../../utils/timer.h"

#if _OPENMP
#include "common/flat_init.h"
#else
#include "common/flat_init.h"
#endif
#include "../../utils/timer.h"

#if USE_SILLYMP
#include "sillymp.h"
#endif

static const integer ONE = 1;
static const integer MINUS_ONE = -1;
static const integer ZERO = 0;

static const double dzero = 0.0;
static const double done = 1.0;
static const double dminusone = -1.0;


void cholesky(integer N, integer BS, double A[N][N]) {
	integer INFO;
	
	lapack_dpotrf("L", &N, A, &N, &INFO);
}


//--------------------------------------------------------------------------------

static long N_;


static inline double initializeElement(long I, long J, long i, long j) {
	// Symetric and positive definite
	if ((I+i) == (J+j)) {
		return ((double) N_) + (I+J+i*3.0+j*5.0)/(10.0*N_);
	} else {
		return (I+J+i*3.0+j*5.0)/(10.0*N_);
	}
}


static void init(integer N, integer BS, double A[N][N]) {
#if USE_SILLYMP
	sillymp_init(atoi(getenv("OMP_NUM_THREADS")));
#endif
	
	// fill the matrix with random values
	initializeUsingUserSpecification(N, N, BS, BS, A);
	
#if USE_SILLYMP
	sillymp_end();
#endif
	force_blas_init();
}


int main(int argc, char *argv[]) {
	if (argc != 7) {
		fprintf(stderr, "Usage: %s <N> <N> <block size> <block size> <index> <I|H|V|B|PW>\n", argv[0]);
		exit (1);
	}
	
	integer N = atol(argv[1]);
	integer BS = atol(argv[3]);
	parseInitializationType(argv[6]);
	
	N_ = N;
	
	double (*A)[N] = (double *) malloc (sizeof(double) * N * N);
	
	init(N, BS, A);
	
	START_TIME();
	cholesky(N, BS, A);
	STOP_TIME();
	
	if (getenv("MEASURE_TIME") == NULL) {
		printf("%f", (0.33*N*N*N+0.5*N*N+0.17*N)/GET_TIME() );
	} else {
		printf("%f", GET_TIME() / 1000000.0);
	}
	
	return 0;
}
