#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdint.h>
#include <unistd.h>

#include <complex.h>
#ifdef I
#undef I
#endif

#include <fftw3.h>


#define TYPE double _Complex
#include "../../utils/timer.h"

#if _OPENMP
#include "common/flat_init.h"
#else
#include "common/flat_init.h"
#endif
#include "../../utils/timer.h"

#if USE_SILLYMP
#include "sillymp.h"
#endif


#define WISDOM_FILE \
	(getenv("FFTW_WISDOM_FILE") != 0 ? getenv("FFTW_WISDOM_FILE") : "fftw_threads.wisdom")


fftw_plan plan;


static long elements_per_core;


void prefault(void *a) {
	memset(a, 0, elements_per_core * sizeof(double _Complex));
}

void init(void *a) {
	double _Complex *A = (double _Complex *)a;
	
	for (long i=0; i<elements_per_core; i++) {
		A[i] = (double)(i) + _Complex_I * (-i);
	}
}


static inline double _Complex initializeElement(long I, long J, long i, long j) {
	return I+i+j;
}


int main(int argc, char **argv) {
	double _Complex *A;
	
	if (argc != 7) {
		fprintf(stderr, "Usage: %s <N> <N> <init BS> <init BS> <index> <I|H|V|B|PW|SEQ>\n", argv[0]);
		return 1;
	}
	
	long N_SQRT = atol(argv[1]);
	long INIT_BS = atol(argv[3]);
	int nthreads = atoi(argv[5]);
	parseInitializationType(argv[6]);
	
	long N = N_SQRT*N_SQRT;
	
	fftw_init_threads();
	fftw_plan_with_nthreads(nthreads);
	
	
	A = (double _Complex *)malloc( sizeof(double _Complex) * N );
	if (A == NULL) {
		fprintf(stderr, "Error: memory allocation failed.\n");
		exit(2);
	}
	
	// Pre-fault the memory
#if USE_SILLYMP
	sillymp_init(atoi(getenv("OMP_NUM_THREADS")));
#endif
	initializeUsingUserSpecification(N_SQRT, N_SQRT, INIT_BS, INIT_BS, (void *) A);
#if USE_SILLYMP
	sillymp_end();
#endif
	
	// Load the wisdom
	FILE *file = fopen(WISDOM_FILE, "r");
	if (file != NULL) {
		fftw_import_wisdom_from_file(file);
		fclose(file);
	}
	
	fftw_iodim64 dims[] = {N, 1, 1};
	fftw_iodim64 howmany[] = {1, N * sizeof (double _Complex), N * sizeof (double _Complex)};
	
	fftw_plan plan = fftw_plan_guru64_dft(1, dims, 1, howmany, A, A, FFTW_FORWARD, FFTW_PATIENT|FFTW_NO_NONTHREADED);
	
	// Initialize the data
#if USE_SILLYMP
	sillymp_init(atoi(getenv("OMP_NUM_THREADS")));
#endif
	initializeUsingUserSpecification(N_SQRT, N_SQRT, INIT_BS, INIT_BS, (void *) A);
#if USE_SILLYMP
	sillymp_end();
#endif
	
	// Execute
	START_TIME();
	fftw_execute(plan);
	STOP_TIME();
	
	if (getenv("MEASURE_TIME") == NULL) {
		printf("%f", (5.0*(double)N*log(N)/log(2.0))/GET_TIME());
	} else {
		printf("%f", GET_TIME() / 1000000.0);
	}
	
	// Save the wisdom
	file = fopen(WISDOM_FILE, "w");
	fftw_export_wisdom_to_file(file);
	fclose(file);
	
	fftw_destroy_plan(plan);
	fftw_cleanup_threads();
	
	return 0;
}

